/*****************************************/
/**             CONJUNTOS               **/
/*****************************************/
param NB>=0, integer;
set Bloques := 1..NB;
param NP>=0, integer;
set Proponentes := 1..NP;
param NO>=0, integer;
set Ofertas := 1..NO;
param NL>=0, integer;
set Ligas := 1..NL;

/*****************************************/
/**              GENERAL                **/
/*****************************************/
param EpsPrecio, default 0.001 ;             #(USD/MWh)

/*****************************************/
/**             OFERTAS                 **/
/*****************************************/
param IdentificadorBloque{i in Bloques}, symbolic ;
param EnergiaBloque{i in Bloques};           #(GWh)
param NumeroSubbloques{i in Bloques};        #(-)
param PrecioReserva{i in Bloques};           #(USD/MWh)
param IdentificadorProponente{i in Proponentes}, symbolic ;
param IdentificadorOferta{i in Ofertas}, symbolic ;
param IdBloque{i in Ofertas}, >=1, <=NB;     #(-)
param IdProponente{i in Ofertas}, >=1, <=NP; #(-)
param NminSubbloques{i in Ofertas}, >=1;     #(-)
param NmaxSubbloques{i in Ofertas}, >=1;     #(-)
param PrecioNiveladoOferta{i in Ofertas};    #(USD/MWh)
param IdOfertaI{i in Ligas}, >=1;            #(-)
param IdOfertaJ{i in Ligas}, >=1;            #(-)

param EnergiaTotal, := sum{i in Bloques} EnergiaBloque[i];         #(GWh)
param PrecioHolgura{i in Bloques} := PrecioReserva[i]+EpsPrecio ;  #(USD/MWh)

/*****************************************/
/**       VARIABLES DE DECISION         **/
/*****************************************/
var x{i in Ofertas}, integer ;  # adjudicacion ofertas (subbloques)
var z{i in Ofertas}, binary ;   # adjudicacion ofertas (si/no)

/*****************************************/
/**              OBJETIVO               **/
/*****************************************/
# Precio nivelado medio ponderado
minimize Precio: 
        sum{i in Bloques} PrecioHolgura[i]*EnergiaBloque[i]/EnergiaTotal
        + sum{i in Bloques, j in Ofertas: IdBloque[j]==i} (EnergiaBloque[i]/NumeroSubbloques[i]/EnergiaTotal)*(PrecioNiveladoOferta[j]-PrecioHolgura[i])*x[j] ;
                
/*****************************************/
/**           RESTRICCIONES             **/
/*****************************************/
subject to HolguraPositiva{i in Bloques}: 
        sum{j in Ofertas: IdBloque[j]==i} x[j] <= NumeroSubbloques[i] ;  
subject to Semicontinuidad1{i in Ofertas}: 
        x[i] >= NminSubbloques[i]*z[i] ;
subject to Semicontinuidad2{i in Ofertas}: 
        x[i] <= NmaxSubbloques[i]*z[i] ;
subject to IgualdadSubbloquesAdjudicados{i in 1..NB-1}: 
        sum{j in Ofertas: IdBloque[j]==i} x[j] == sum{j in Ofertas: IdBloque[j]==i+1} x[j] ;
subject to LigasOfertas1{i in Ligas}: 
        z[IdOfertaI[i]] - z[IdOfertaJ[i]] == 0 ;
subject to LigasOfertas2{i in Ligas}: 
        NmaxSubbloques[IdOfertaI[i]]*z[IdOfertaI[i]] - x[IdOfertaI[i]] == NmaxSubbloques[IdOfertaJ[i]]*z[IdOfertaJ[i]] - x[IdOfertaJ[i]]  ;

/*****************************************/
/**           OPTIMIZACION              **/
/*****************************************/
solve;
param y{i in Bloques} := NumeroSubbloques[i] - sum{j in Ofertas: IdBloque[j]==i} x[j] ;
param PNMP := sum{i in Bloques} ( EnergiaBloque[i]/NumeroSubbloques[i]/EnergiaTotal ) * 
                                    ( PrecioHolgura[i]*y[i] + sum{j in Ofertas: IdBloque[j]==i}PrecioNiveladoOferta[j]*x[j] );
printf "\nPNMP = %.6f\n\n", PNMP ;
printf ": PrecioHolgura NumeroSubbloques   y     :=\n" ;
for {i in Bloques}
{ 
    printf "%d\t %.3f\t\t %d\t %d\n", i, PrecioHolgura[i], NumeroSubbloques[i], y[i];
}
printf ";\n\n" ;
printf ":  PrecioNiveladoOferta NminSubbloques NmaxSubbloques   x     :=\n" ;
for {i in Ofertas}
{ 
    printf "%d\t %.3f\t\t\t %d\t\t %d\t %d\n", i, PrecioNiveladoOferta[i], NminSubbloques[i], NmaxSubbloques[i], x[i];
}
printf ";\n\n" ;

/*****************************************/
/**        ARCHIVOS DE SALIDA           **/
/*****************************************/
param DEL symbolic := "," ;
param csvFile symbolic := "LIC.csv" ;
printf "\n" > csvFile ; 
printf "PNMP%s %.6f\n", DEL, PNMP >> csvFile ; 
printf "\n" >> csvFile ; 
#---------------------------------------------
printf "BLOQUES\n" >> csvFile ; 
printf "Id%s Bloque%s Energia%s Subbloques%s Precio reserva%s Holgura\n", DEL, DEL, DEL, DEL, DEL >> csvFile ; 
for {i in Bloques} {
    printf "%d%s ", i, DEL >> csvFile ; 
    printf "%s%s ", IdentificadorBloque[i], DEL >> csvFile ; 
    printf "%.6f%s ", EnergiaBloque[i], DEL >> csvFile ; 
    printf "%d%s ", NumeroSubbloques[i], DEL >> csvFile ; 
    printf "%.6f%s ", PrecioReserva[i], DEL >> csvFile ; 
    printf "%d\n", y[i] >> csvFile ; 
}
printf "\n" >> csvFile ; 
#---------------------------------------------
printf "OFERTAS\n" >> csvFile ; 
printf "Id%s Oferta%s Proponente%s ", DEL, DEL, DEL >> csvFile ; 
printf "Bloque%s Nro minimo subbloques%s ", DEL, DEL >> csvFile ; 
printf "Nro maximo subbloques%s Precio nivelado%s Adjudicacion\n", DEL, DEL >> csvFile ; 
for {i in Ofertas} {
    printf "%d%s ", i, DEL >> csvFile ; 
    printf "%s%s ", IdentificadorOferta[i], DEL >> csvFile ; 
    printf "%s%s ", IdentificadorProponente[IdProponente[i]], DEL >> csvFile ; 
    printf "%s%s ", IdentificadorBloque[IdBloque[i]], DEL >> csvFile ; 
    printf "%d%s ", NminSubbloques[i], DEL >> csvFile ; 
    printf "%d%s ", NmaxSubbloques[i], DEL >> csvFile ; 
    printf "%.6f%s ", PrecioNiveladoOferta[i], DEL >> csvFile ; 
    printf "%d\n", x[i] >> csvFile ; 
}
printf "\n" >> csvFile ; 
#---------------------------------------------



end;
